unit testPdfColorsUnit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, AdvUtil, Vcl.Grids, AdvObj, BaseGrid,
  AdvGrid, Vcl.StdCtrls, AdvCustomComponent, AdvPDFIO, AdvGridPDFIO, HTMLText;

type
  TForm1 = class(TForm)
    AdvStringGrid1: TAdvStringGrid;
    AdvGridPDFIO1: TAdvGridPDFIO;
    cbIndex: TCheckBox;
    cbColor: TCheckBox;
    cbName: TCheckBox;
    cbOrig: TCheckBox;
    cbCurrent: TCheckBox;
    cbDiff: TCheckBox;
    pdfBtn: TButton;
    htmlBtn: TButton;
    HTMLStaticText1: THTMLStaticText;
    procedure FormCreate(Sender: TObject);
    procedure AdvStringGrid1GetCellColor(Sender: TObject; ARow, ACol: Integer;
      AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
    procedure onPdfClick(Sender: TObject);
    procedure onClickToggleCol(Sender: TObject);
    procedure onHtmlClick(Sender: TObject);
  private
    function cbStates : string;
    function tempFile( const basename:string ) : string;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses ShellApi;

{$R *.dfm}

const COLTITLE_INDEX = 'index';
      COLTITLE_COLOR = 'color';
      COLTITLE_PLAYER = 'name';
      COLTITLE_TEAM   = 'team';
      COLTITLE_ORIG   = 'orig';
      COLTITLE_CURR   = 'curr';
      COLTITLE_CHANGE = 'diff';

procedure TForm1.FormCreate(Sender: TObject);
const DUMMY_DATA : Array of String = [
        '' + COLTITLE_INDEX  + ','
           + COLTITLE_COLOR  + ','
           + COLTITLE_PLAYER + ','
           + COLTITLE_TEAM   + ','
           + COLTITLE_ORIG   + ','
           + COLTITLE_CURR   + ','
           + COLTITLE_CHANGE + '',
        '3,13299711,"Joshua",Team-Z,5.3,5.3,0',
        '65,14730722,"Fred",Team-P,7.3,7.0,-0.3',
        '87,12185009,"Ted",Team-W,5.3,5.4,0.1',
        '1,16704726,"Chris",Team-B,6.9,7,0.1',
        '69,11651309,"Steve",Team-Q,5.2,5.4,0.2',
        '8,15395583,"Conrado",Team-S,8.4,8.3,-0.1',
        '33,14155762,"William",Team-L,3.9,4,0.1'
       ];
var sList : TStringList;
   csvFile, s : string;
begin
  // Populate grid.
  sList := TStringList.Create;
  for s in DUMMY_DATA do sList.Add(s);
  csvFile := tempFile( 'sampleDataForPdfColorTesting.csv' );
  try
    sList.SaveToFile( csvFile );
    AdvStringGrid1.LoadFromCSV( csvFile );
  finally
    DeleteFile( csvFile );
    sList.Free;
  end;

  // Syncronize checkbox captions to column headers.
  cbIndex.Caption := COLTITLE_INDEX;
  cbColor.Caption := COLTITLE_COLOR;
  cbName.Caption := COLTITLE_PLAYER;
  cbOrig.Caption := COLTITLE_ORIG;
  cbCurrent.Caption := COLTITLE_CURR;
  cbDiff.Caption := COLTITLE_CHANGE;
end;

function TForm1.cbStates : string;
  function cbState( const cb : TCheckbox ) : string;
  begin
    Result := cb.Caption + '[';
    if cb.Checked then Result := Result + 'X' else Result := Result + ' ';
    Result := Result + ']  ';
  end;
begin
  Result := cbState(cbIndex)+cbState(cbColor)+cbState(cbName)+cbState(cbOrig)+cbState(cbCurrent)+cbState(cbDiff);
end;

function TForm1.tempFile( const basename:string ) : string;
var tempFolder: array[0..MAX_PATH] of Char;
begin
  GetTempPath(MAX_PATH, @tempFolder);  // trailing backslash is included here.
  Result := StrPas(tempFolder) + basename;
end;

procedure TForm1.onClickToggleCol(Sender: TObject);
var col : integer;
begin
  col := AdvStringGrid1.ColumnByHeader( (Sender as TCheckbox).Caption );

  if (Sender as TCheckbox).Checked
     then AdvStringGrid1.UnHideColumn( col )
     else AdvStringGrid1.HideColumn( col );
end;

procedure TForm1.AdvStringGrid1GetCellColor(Sender: TObject; ARow,
  ACol: Integer; AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
var colTitle, valStr : string;
    rankDiff : single;
    teamColorColIndex : integer;
    teamColorStr : string;
    teamColorInt : integer;
begin
  if ARow=0 then
     begin
       AFont.Style := [fsBold];
       Exit;
     end;

  colTitle := AdvStringGrid1.Cells[ ACol, 0 ];

  if colTitle=COLTITLE_PLAYER then
      begin
        AFont.Style := [fsBold];
      end
  else if colTitle=COLTITLE_TEAM then
      begin
        AFont.Style := [fsBold];

        teamColorColIndex := AdvStringGrid1.ColumnByHeader( COLTITLE_COLOR );
        teamColorStr := AdvStringGrid1.Cells[teamColorColIndex, ARow];
        teamColorInt := StrToInt( teamColorStr );
        ABrush.Color := TColor( teamColorInt );
      end
  else if colTitle=COLTITLE_ORIG then
      begin
        AFont.Style := [fsItalic];
      end
  else if colTitle=COLTITLE_CHANGE then
      begin
        valStr := AdvStringGrid1.Cells[ ACol, ARow ];
        rankDiff := StrToFloatDef( valStr, 0 );
             if rankDiff<0 then AFont.Color := clRed
        else if rankDiff>0 then AFont.Color := clGreen
                           else AFont.Color := clGray;
      end;
end;

procedure TForm1.onPdfClick(Sender: TObject);
var tempPdfFile : string;
begin
  tempPdfFile := tempFile( 'sampleColorTestOutput.pdf' );
  AdvGridPDFIO1.Grid := AdvStringGrid1;
  AdvGridPDFIO1.Options.Header := cbStates;
  AdvGridPDFIO1.Options.OpenInPDFReader := TRUE;
  AdvGridPDFIO1.Save( tempPdfFile );
end;

procedure TForm1.onHtmlClick(Sender: TObject);
var tempPdfFile : string;
begin
  tempPdfFile := tempFile( 'sampleColorTestOutput.html' );
  AdvStringGrid1.SaveToHtml( tempPdfFile );
  ShellExecute(Handle, nil, PChar(tempPdfFile), nil,  nil, SW_SHOWNORMAL);
end;

end.
